#
# Copyright (C) 2008, 2012 Fabian Knittel <fabian.knittel@lettink.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import mimetypes
import elixir
from oe_mailmodel import DeliveredMail, DeliveredAttachment
import email.header
import email.utils
import datetime
import re
import codecs
import os.path


def try_decoding(str, codecs):
    for codec in codecs:
        try:
            return unicode(str, codec)
        except UnicodeDecodeError:
            pass
    return None

def header_as_unicode(header):
    if isinstance(header, unicode):
        return header

    full_str = ''
    for header_str, header_enc in email.header.decode_header(header):
        if header_enc is not None:
            decoder = codecs.getdecoder(header_enc)
            header_str = decoder(header_str)[0]
        else:
            header_str = try_decoding(header_str, ['ascii', 'iso-8859-15',
                'utf-8'])
            if header_str is None:
                print 'Warning: Could not decode %s' % repr(str)

        # decode_header strips spaces, so we strip everything too and add a
        # single space to seperate the strings. Might go wrong sometimes, but
        # it's correct more often than without space.
        full_str += header_str.strip() + u' '
    return full_str.strip()

def name_is_taken(msg_atts, name):
    for att_name, att_payload in msg_atts:
        if att_name == name:
            return True
    return False

def make_name_unique(msg_atts, att_name):
    root, ext = os.path.splitext(att_name)

    unique_name = att_name
    cnt = 2
    # Loop until the attachment name is unique.
    while name_is_taken(msg_atts, unique_name):
        unique_name = "%s_%d%s" % (root, cnt, ext)
        cnt += 1
    return unique_name

def is_attachment(msg_part):
    """Returns `True` if the message part is an attachment.  Relies on the MUA
    to properly indicate attachments.
    """
    missing = object()
    att_flag = msg_part.get_param('attachment', missing, 'content-disposition')
    return att_flag is not missing

def handle_message(message, trust_date=False):
    msg_subj = header_as_unicode(message.get_all('subject')[0])
    msg_from = header_as_unicode(message.get_all('from')[0])

    msg_atts = []
    for msg_part in message.walk():
        # Skip message bodies.
        if msg_part.is_multipart() or not is_attachment(msg_part):
            continue

        # Get an attachment name.
        att_name = msg_part.get_filename(None)
        if att_name:
            att_name = header_as_unicode(att_name)
        if not att_name or att_name == u'':
            ext = mimetypes.guess_extension(msg_part.get_content_type())
            att_name = u'unbekannt%s' % ext

        # Skip boring attachments.
        if att_name in [u'smime.p7s', u'signature.asc', u'unbekannt.pgp']:
            continue

        # Make attachment names unique per mail.
        att_name = make_name_unique(msg_atts, att_name)

        # Add attachment name and content to the list ...
        pl = msg_part.get_payload(decode=True)
        msg_atts.append((att_name, pl))

    # We only want to store mails with attachments.
    if msg_atts == []:
        return False

    # Store the mail.
    mail = DeliveredMail(sender=msg_from, subject=msg_subj)
    if trust_date:
        mail.date = datetime.datetime.fromtimestamp(email.utils.mktime_tz(
                email.utils.parsedate_tz(message.get_all('date')[0])))
    for att_name, att_payload in msg_atts:
        mail.attachments.append(DeliveredAttachment(name=att_name,
                data=att_payload))
    elixir.session.commit()
    return True

# vim:set textwidth=80 tabstop=8 softtabstop=4 shiftwidth=4 et:
